<?php

/// Parse style/blog_or_admin.css to find the fields indicating elements widths,
/// and fill them accordingly in style/blog_or_admin.js

class ReinitJsvars {
  
  // A few helpers for parsing

  private static function isGap($c) {
    return ($c == ' ' || $c == '\t' || $c == '\n');
  }

  private static function isDigit($c) {
    return ($c >= '0' && $c <= '9');
  }

  // Main function:

  public static function Run($mode) {
    
    // This file will be used:
    if ($mode == "blog") $stylesheet = implode("", file("../../style/shared.css", 'r'));
    else if ($mode == "admin") $stylesheet = implode("", file("../style/shared.css", 'r'));
    
    // To write to:
    if ($mode == "blog") $writer = fopen("../../style/variables.js", 'w');
    else if ($mode == "admin") $writer = fopen("../style/variables.js", 'w');
    
    // Some comments for human reading first:
    fwrite($writer,"<script type='text/javascript'>
/* Styles indicating document width(s) are reported here,
 * because they are needed to (re)compute elements size after window resizing.
 * All values in pixels.
 * -----
 * tml, tmr = text margin left/right
 * tpl, tpr = text padding left/right
 * mml, mmr = menu margin left/right
 * mpl, mpr = menu padding left/right
 * tbw, mbw = text/menu border width
 */\n");
    
    // remove all comments
    $lastEndComment = 0;
    while (TRUE) {
      $beginComment = strpos($stylesheet, "/*", $lastEndComment);
      if ($beginComment == FALSE) break;
      $endComment = strpos($stylesheet, "*/", $beginComment);
      if ($endComment == FALSE) {
        // may happen if user forget to close last comments...
        $endComment = strlen($stylesheet);
      }
      $beforeComment = substr($stylesheet, $lastEndComment, $beginComment - $lastEndComment);
      $afterComment = substr($stylesheet, $endComment, strlen($stylesheet) - $endComment);
      $stylesheet = $beforeComment.$afterComment;
      $lastEndComment = $beginComment;
    }
    
    // search for div#innerText[SPACINGS]{
    $lastInnerTextIndex = 0;
    while (TRUE) {
      $innerText = strpos($stylesheet, "div#innerText", $lastInnerTextIndex);
      if ($innerText == FALSE) break;
      $startIndex = $innerText + 13;
      while (ReinitJsvars::isGap($stylesheet[$startIndex])) $startIndex++;
      if ($stylesheet[$startIndex] == '{') { // otherwise it's a nested specif inside div#innerText
        // process all potential fields:
        $spacings = ReinitJsvars::getSpacings($stylesheet, $startIndex+1);
        fwrite($writer, "var tml = ".$spacings["ml"].";\n");
        fwrite($writer, "var tmr = ".$spacings["mr"].";\n");
        fwrite($writer, "var tpl = ".$spacings["pl"].";\n");
        fwrite($writer, "var tpr = ".$spacings["pr"].";\n");
        fwrite($writer, "var tbw = ".$spacings["bw"].";\n");
        break;
      }
      $lastInnerTextIndex = $startIndex;
    }
    
    // search for div#innerMenu[SPACINGS]{
    $lastInnerMenuIndex = 0;
    while (TRUE) {
      $innerMenu = strpos($stylesheet, "div#innerMenu", $lastInnerMenuIndex);
      if ($innerMenu == FALSE) break;
      $startIndex = $innerMenu + 13;
      while (ReinitJsvars::isGap($stylesheet[$startIndex])) $startIndex++;
      if ($stylesheet[$startIndex] == '{') { // otherwise it's a nested specif inside div#innerMenu
        // process all potential fields:
        $spacings = ReinitJsvars::getSpacings($stylesheet, $startIndex+1);
        fwrite($writer, "var mml = ".$spacings["ml"].";\n");
        fwrite($writer, "var mmr = ".$spacings["mr"].";\n");
        fwrite($writer, "var mpl = ".$spacings["pl"].";\n");
        fwrite($writer, "var mpr = ".$spacings["pr"].";\n");
        fwrite($writer, "var mbw = ".$spacings["bw"].";\n");
        break;
      }
      $lastInnerMenuIndex = $startIndex;
    }
    
    fwrite($writer, "</script>\n");
    fclose($writer);
  }

  // Get values of the specified variables, and then write them to the elements/styles.js file
  private static function getSpacings($stylesheet, $startIndex) {
    
    $spacings = array(
      "ml" => 0,
      "mr" => 0,
      "pl" => 0,
      "pr" => 0,
      "bw" => 0);
    
    // extract substring of interest:
    $endIndex = strpos($stylesheet, "}", $startIndex);
    // append a ';' in case of:
    $styleSpecif = substr($stylesheet, $startIndex, $endIndex - $startIndex).";";
    
    // margin
    
    $startIndex = strpos($styleSpecif, "margin");
    if ($startIndex != FALSE) {
      $numbers = ReinitJsvars::readNumbers($styleSpecif, $startIndex + 6);
      if (count($numbers) == 1) {
        $spacings["ml"] = $numbers[0];
        $spacings["mr"] = $numbers[0];
      }
      else {
        $spacings["ml"] = $numbers[3];
        $spacings["mr"] = $numbers[1];
      }
    }
    $startIndex = strpos($styleSpecif, "margin-left");
    if ($startIndex != FALSE) {
      $numbers = ReinitJsvars::readNumbers($styleSpecif, $startIndex + 11);
      $spacings["ml"] = $numbers[0];
    }
    $startIndex = strpos($styleSpecif, "margin-right");
    if ($startIndex != FALSE) {
      $numbers = ReinitJsvars::readNumbers($styleSpecif, $startIndex + 12);
      $spacings["mr"] = $numbers[0];
    }
    
    // padding
    
    $startIndex = strpos($styleSpecif, "padding");
    if ($startIndex != FALSE) {
      $numbers = ReinitJsvars::readNumbers($styleSpecif, $startIndex + 7);
      if (count($numbers) == 1) {
        $spacings["pl"] = $numbers[0];
        $spacings["pr"] = $numbers[0];
      }
      else {
        $spacings["pl"] = $numbers[3];
        $spacings["pr"] = $numbers[1];
      }
    }
    $startIndex = strpos($styleSpecif, "padding-left");
    if ($startIndex != FALSE) {
      $numbers = ReinitJsvars::readNumbers($styleSpecif, $startIndex + 12);
      $spacings["pl"] = $numbers[0];
    }
    $startIndex = strpos($styleSpecif, "padding-right");
    if ($startIndex != FALSE) {
      $numbers = ReinitJsvars::readNumbers($styleSpecif, $startIndex + 13);
      $spacings["pr"] = $numbers[0];
    }
    
    // border width
    
    $startIndex = strpos($styleSpecif, "border");
    if ($startIndex != FALSE) {
      $numbers = ReinitJsvars::readNumbers($styleSpecif, $startIndex + 6);
      $spacings["bw"] = $numbers[0];
    }
    $startIndex = strpos($styleSpecif, "border-width");
    if ($startIndex != FALSE) {
      $numbers = ReinitJsvars::readNumbers($styleSpecif, $startIndex + 12);
      $spacings["bw"] = $numbers[0];
    }
    
    return $spacings;
  }

  private static function readNumbers($styleSpecif, $startIndex) {
    
    $numbers = array();
    
    $endIndex = strpos($styleSpecif, ";"); // we know that there is ';' (may be artificially added)
    $token = strtok(substr($styleSpecif, $startIndex, $endIndex - $startIndex), " \t\n");
    while ($token !== FALSE) {
      $lastDigitIndex = 0;
      while ($lastDigitIndex < strlen($token) && ReinitJsvars::isDigit($token[$lastDigitIndex])) {
        $lastDigitIndex++;
      }
      $numbers[] = intval(substr($token, 0, $lastDigitIndex));
      $token = strtok(" \t\n");
    }
    
    return $numbers;
  }
}

?>
