<?php

/// Permalink for any blog entry

require_once("./utils/entities/post.php");
require_once("./utils/entities/url_params.php");
require_once("./utils/entities/comment.php");
require_once("./utils/entities/follower.php");
require_once("./utils/entities/trackback.php");
require_once("./utils/text_editor/public_buttons.php");

$url = $_GET["id"];

// get post infos:
$post = Post::fromFile("./".DataUrlParams::fromUrl($url, ObjType::POST)->toUrl(UrlType::FILE));
$withMath = $post->hasMath();
$withCode = $post->hasCode();

// get comments infos:
$dir = opendir("./".DataUrlParams::fromUrl($url, ObjType::COMMENT)->toUrl(UrlType::FOLDER));
$comments = array();
while (($file = readdir($dir)) !== FALSE) {
  if ($file[0] == '.') continue;
  $comment = Comment::fromFile($commentsFolder.$file);
  $withMath |= $comment->hasMath();
  $withCode |= $comment->hasCode();
  $comments[] = $comment;
}
closedir($dir);

// also look 'flags' (not very good design here..)
$flags = fopen("./elems/flags", 'r');
$withCode |= substr(fgets($flags), 0, -1) == "true";
$withMath |= substr(fgets($flags), 0, -1) == "true";
fclose($flags);

// get trackbacks infos:
$dir = opendir("./".DataUrlParams::fromUrl($url, ObjType::TRACKBACK)->toUrl(UrlType::FOLDER));
$trackbacks = array();
while (($file = readdir($dir)) !== FALSE) {
  if ($file[0] != '.') {
    $trackbacks[] = Trackback::fromFile($trackbacksFolder.$file);
  }
}
closedir($dir);

// need absolute path to blog:
$protocol = isset($_SERVER['HTTPS']) ? "https" : "http";
$serverAddress = $protocol."://".$_SERVER['HTTP_HOST'];
$absolutePathToBlog = $serverAddress.$_SERVER['PHP_SELF']; // unpruned
$absolutePathToBlog = substr($absolutePathToBlog, 0, strrpos($absolutePathToBlog, "/"))."/";

$trackbackLink = $absolutePathToBlog.TrackbackUrlParams::fromUrl($url)->toUrl();
?>

<html>

<head>

  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
  <meta name="description" content="<?php echo $post->getTitle(); ?>"/>
  <meta name="keywords" content="<?php echo $post->getKeywords(); ?>"/>

  <title><?php echo $post->getTitle(); ?></title>
  
  <link href="./style/shared.css" rel="stylesheet" type="text/css"/>
  <link href="./style/index.css" rel="stylesheet" type="text/css"/>
  <link href="./style/arxiv.css" rel="stylesheet" type="text/css"/>
  <link href="./style/post.css" rel="stylesheet" type="text/css"/>
  <link href="./style/comment.css" rel="stylesheet" type="text/css"/>
  <link href="./style/trackback.css" rel="stylesheet" type="text/css"/>
  <link href="./style/text_editor.css" rel="stylesheet" type="text/css"/>
  <link href='./style/core_text.css' rel='stylesheet' type='text/css'/>
<?php
if ($withCode) {
  echo "<link href='./style/code.css' rel='stylesheet' type='text/css'/>";
}
?>

  <?php include("./style/variables.js"); ?>
  <script src="./javas/widths.js" type="text/javascript"></script>
  <script src="./javas/time.js" type="text/javascript"></script>
  <script src="./javas/toggle_menu.js" type="text/javascript"></script>
  <script src="./follow_thread/subscribe.ajax.js" type="text/javascript"></script>
  <script src="./comment/insert.ajax.js" type='text/javascript'></script>
  <script src="./utils/text_editor/actions.js" type='text/javascript'></script>
<?php
if ($withMath) {
  echo "<script src='http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML' type='text/javascript'></script>\n";
}
?>

</head>

<body onLoad="document.getElementById('menu').style.display='block';adjustWidths();document.onkeydown=handleKeys">

<div id="main">
  
  <div id="title">
    <span id="mainTitle"> <?php include("./elems/title"); ?> </span><br/>
    <span id="mainSubtitle"> <?php include("./elems/subtitle"); ?> </span>
  </div>
  
  <div id="text"> 

    <div id="innerText">
    
      <div id="posts">
      <?php echo $post->toHtml(); ?>
      </div>
      
      <div id="comments">

        <h4>Comments</h4>
<?php
// print comments (if any)
for ($i=0; $i<count($comments); $i++) echo $comments[$i]->toHtml();
?>
        
        <!-- artificial, to add a comment at the end -->
        <div class="clearer" id="endOfComments"> </div>

      </div>
      
      <div id="newComment">

        <form id="newCommentForm" name="newCommentForm" method="post" action="./comment/preview.php" target="target" 
        onsubmit="window.open('', 'target', 'width=600,height=500,left=100,top=100,status=0,menubar=0,toolbar=0')">

          <p>
            <label for="newCommentUrl">URL (optional)</label>
            <input type="text" class="textField" id="authorUrl" name="authorUrl"/>
          </p>
          
          <p>
            <label for="newCommentAuthor">Author (required)</label>
            <input type="text" class="textField" id="author" name="author"/>
          </p>
          
          <p>
            <label for="commentText">Text (required)</label>
          </p>
          
          <div id="editorButtonsContainer">
            <div id="editorButtons">
              <?php showEditorButtons("./", "commentText"); ?>
            </div>
          </div>
          
          <p>
            <textarea id="commentText" name="commentText" cols="40" rows="10"></textarea>
          </p>
          
          <p>
            <label for="hasMathBox">Has math ?</label>
            <input id="hasMathBox" name="hasMathBox" type="checkbox" value="has math"/><br/>
            <label for="hasCodeBox">Has code ?</label>
            <input id="hasCodeBox" name="hasCodeBox" type="checkbox" value="has code"/>
          </p>
          
          <p>
            <input type="submit" class="validateButton" id="preview" value="Preview"/>
            <input type="button" class="validateButton" id="submit" value="Send" onClick="insertComment('<?php echo $url; ?>')"/>
          </p>
          
        </form>
        
        <p>
          <span id="commentWarner" class="warner">Placeholder.</span>
        </p>
        
      </div>
    
      <div id="trackbacks">
      
<?php
// print trackbacks (if any)
if (count($trackbacks) > 0) {
  echo "<h4>Comments</h4>\n";
}
for ($i=0; $i<count($trackbacks); $i++) echo $trackbacks[$i]->toHtml();
// set hook for other blogs to find us:
echo Trackback::rdfAutodiscover($post->getTitle(), $post->getPermalink(), $trackbackLink);
?>
    
      </div>
    
      <div id="followThread">

        <form id="followThreadForm" name="followThreadForm">

          <p>
            <label for="email">Fill e-mail to receive updates</label><br/>
            <input type="text" class="textField" id="email" name="email"/>
          </p>

          <p>
            <input type="button" class="validateButton" value="Subscribe" onClick="subscribe('<?php echo $url; ?>')"/>
          </p>

        </form>

        <p>
          <span id="subscribeWarner" class="warner">Placeholder.</span>
        </p>

        <p>
          <span id="subscribeNotifier" class="notifier">Placeholder.</span>
        </p>

      </div>
    
    </div>
      
  </div>
   
  <div id="menu"> 
      
    <div id="innerMenu">
      
      <div id="menuLinks">
        <p>
          <a href="./rss_feed/rss.xml">RSS</a>
          <a href="./rss_feed/rss.xml">
            <img src="./rss_feed/icon.png" alt="RSS_icon" width="15px"/>
          </a>
        </p>
        <hr class="hrMenu"/>
        <p>
          Toggle me: 'ctrl+alt+M'
        </p>
        <p>
          <input type="checkbox" id="resizeCheckbox" checked="checked"/>
          <label for="resizeCheckbox">auto</label>
          <input type="button" id="resizeButton" value="resize" onClick="adjustWidths()"/>
        </p>
        <hr class="hrMenu"/>
        <?php include("./elems/menu"); ?>
      </div>
      
    </div>
      
  </div>
    
  <div id="footer">
  <?php include("./elems/footer"); ?>
  </div>
  
  <div class="clearer"> </div> 

</div>

</body>

</html>
