<?php
// I'm a little bit confused about which header should go here.
// See http://php.net/manual/fr/function.header.php...
header("Expires: Mon, 01 Jan 2001 00:00:00 GMT");
header("Last-Modified: ".date("r"));
header("Cache-Control: no-cache, must-revalidate");
header("Pragma: no-cache");

require_once("./admin/access/check_user.php");
require_once("./utils/entities/user.php");

// check if root is defined. if yes, then check user (to prevent easy blog hacking)
$authorsDir = opendir("./admin/users_infos/");
while (($file = readdir($authorsDir)) !== FALSE) {
  if ($file[0] != '.') {
    // some user file is present
    $user = checkUser("./", "../../install.php");
    if (!$user->isRoot()) exit("Operation not allowed.");
    break;
  }
}
closedir($authorsDir);

if (isset($_POST["author"])) {

  $author = $_POST["author"];
  $email = $_POST["email"];
  $encrypted1 = $_POST["encrypted1"];
  $firstName = $_POST["firstname"]; // no check.. can be empty
  $lastName = $_POST["lastname"]; // no check.. can be empty
  $question = $_POST["question"];
  $secret1 = $_POST["secret1"];

  unset($_POST["author"]);
  unset($_POST["email"]);
  unset($_POST["encrypted1"]);
  unset($_POST["firstname"]);
  unset($_POST["lastname"]);
  unset($_POST["question"]);
  unset($_POST["secret1"]);

  if (strlen($author) == 0) exit("Empty user name.");
  if (!filter_var($email, FILTER_VALIDATE_EMAIL)) exit("Invalid email.");

  // generate salt and set encrypted + secret level 2
  $salt = uniqid();
  $encrypted2 = sha1($salt.$encrypted1);
  $secret2 = sha1($salt.$secret1);

  // create file in users_infos (this user is root):
  $rootUser = new User(
    $author, $email, $firstName, $lastName, $salt, $encrypted2, $question, $secret2, TRUE);
  $rootUser->toFile("./admin/users_infos/".$author);

  // also set cookie to avoid login screen next:
  setcookie("verify", $encrypted2, 0, "/");
  
  exit(); // stop at this point
}

// set dir/files permissions [http://php.net/manual/fr/function.chmod.php]
function chmod_R($path, $mode) {
  if (is_dir($path)) {
    chmod($path, $mode);
    $dh = opendir($path);
    while (($file = readdir($dh)) !== FALSE) {
      // skip self and parent pointing directories
      if ($file[0] != '.') {
        chmod_R($path.'/'.$file, $mode);
      }
    }
    closedir($dh);
  }
  else {
    if (!is_link($path)) {
      // links (shortcuts) are skipped
      chmod($path, $mode);
    }
  }
}

// preparation before user enter his informations: set files permissions
chmod_R("./", 0755); // lock everything in writing
chmod_R("./admin", 0711); // only execution rights on admin folder (TODO: or no rights at all ?)
//chmod_R("./utils", 0755); // read+execute on 'utils' folder (TODO: or just read ?)
?>

<!DOCTYPE HTML>

<html>

<head>

  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
  <meta http-equiv="Cache-Control" content="no-cache"> 
  <meta http-equiv="Pragma" content="no-cache"> 
  <meta http-equiv="Expires" content="0">
  
  <title>Blog initial configuration</title>
  
  <link href="./admin/style/shared.css" rel="stylesheet" type="text/css"/>
  <link href="http://fonts.googleapis.com/css?family=Terminal+Dosis+Light|Maven+Pro&v2" 
        rel="stylesheet" type="text/css"/>

  <style>
  div#install {
    margin-left: 100px;
  }
  form#createUserForm {
    width: 70%;
  }
  div#validate {
    margin-left: 100px;
  }
  </style>

  <?php include("./admin/style/variables.js"); ?>
  <script src="./javas/widths.js" type="text/javascript"></script>
  <script src="./javas/sha1.js" type="text/javascript"></script>

  <script type="text/javascript">
  
  function setupUser() {
    
    var form = document.forms.createUserForm;
    var author = form.author.value;
    var email = form.email.value;
    var firstName = form.firstName.value;
    var lastName = form.lastName.value;
    var password1 = form.password1.value;
    var password2 = form.password2.value;
    var question = form.question.value;
    var answer1 = form.answer1.value;
    var answer2 = form.answer2.value;
    var warner = document.getElementById("warner");
    
    if (password1.length == 0 || password1 != password2) {
      warner.innerHTML = "Empty or not matching passwords.";
      warner.style.visibility = "visible";
      return;
    }
    if (answer1.length == 0 || answer1 != answer2) {
      warner.innerHTML = "Empty or not matching answers.";
      warner.style.visibility = "visible";
      return;
    }
    
    var xhr = null; 
    if (window.XMLHttpRequest) xhr = new XMLHttpRequest();
    else if (window.ActiveXObject) { // (old) Internet Explorer 
      try {xhr = new ActiveXObject("Msxml2.XMLHTTP");}
      catch (e) {xhr = new ActiveXObject("Microsoft.XMLHTTP");}
    }
    
    if (xhr == null) {
      alert("Your browser does not support AJAX");
      return;
    }
    
    xhr.onreadystatechange = function() {
      if(xhr.readyState == 4 && xhr.status == 200) {
        if (xhr.responseText != "") {
          warner.innerHTML = xhr.responseText;
          warner.style.visibility = "visible";
        }
        else {
          // relocate to admin panel
          document.location.href = "./admin/";
        }
      }
    };
    
    var encrypted1 = sha1(author + "%" + password1);
    var secret1 = sha1(question + "%" + answer1);
    xhr.open("POST", "./install.php", true);    
    xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    xhr.send("author="+author+"&encrypted1="+encrypted1+"&email="+email+
             "&firstname="+firstName+"&lastname="+lastName+"&secret1="+secret1+"&question="+question);
  }
  </script>

</head>

<body onLoad="adjustWidths();document.getElementById('notifier').style.visibility='visible'">

  <div id="main">
  
    <div id="title">
      <span id="mainTitle">Blog initialization</span>
    </div>
    
    <div id="text"> 

      <div id="innerText">

        <div id="install">
          
          <form id="createUserForm" name="createUserForm">
          
            <p>
              <label for="author">Author name (required)</label><br/>
              <input type="text" class="textField" id="author" name="author" cols="40"/>
              <br/>
              <label for="email">E-mail (required)</label><br/>
              <input type="text" class="textField" id="email" name="email" cols="40"/>
            </p>
            
            <p>
              <label for="firstName">First name (optional)</label><br/>
              <input type="text" class="textField" id="firstName" name="firstName" cols="40"/>
              <br/>
              <label for="lastName">Last name (optional)</label><br/>
              <input type="text" class="textField" id="lastName" name="lastName" cols="40"/>
            </p>
            
            <p>
              <label for="password1" name="password1">Password (required)</label><br/>
              <input type="password" class="passwordField" id="password1" name="password1" cols="40"/>
              <br/>
              <label for="password2" name="password2">Password (again!)</label><br/>
              <input type="password" class="passwordField" id="password2" name="password2" cols="40"/>
            </p>
            
            <p>
              <label for="question">Secret question (required)</label><br/>
              <input type="text" class="textField" id="question" name="question" cols="40"/>
              <br/>
              <label for="answer1">Secret answer (required)</label><br/>
              <input type="password" class="textField" id="answer1" name="answer1" cols="40"/>
              <br/>
              <label for="answer2">Secret answer (again !)</label><br/>
              <input type="password" class="textField" id="answer2" name="answer2" cols="40"/>              
            </p>
          
          </form>
          
        </div>
        
        <div id="validate">
        
          <p>
            <input type="button" class="validateButton" value="Send" onClick="setupUser()"/> 
          </p>
          
          <p>
            <span id="warner" class="warner">Placeholder.</span>
          </p>
          
          <p>
            <span id="notifier" class="notifier">Blog title, menu and footer can be edited on admin page next: click "edit elements" button.</span>
          </p>
          
        </div>
        
      </div>
      
    </div>
    
    <div id="footer">
    <?php include("./admin/elems/footer"); ?>
    </div>
    
    <div id="clearer"> </div> 

  </div>

</body>

</html>
