<?php
define('MAX_RSS_ITEMS', 10);
// WARNING: never touch above line

/// Refresh RSS from admin/thread/post
// NOTE: not using simpleXML and DOM, handling them is not that simple finally...

function rss_refresh($url, $updateMode, $pathToBlog) {
  
  require_once($pathToBlog."utils/entities/post.php");
  require_once($pathToBlog."utils/entities/url_params.php");
  require_once($pathToBlog."utils/entities/user.php");
  include($pathToBlog."rss_feed/last_entries.php");
  
  // quick exit : do nothing if deleted/edited post is too old
  if ($updateMode != UpdateMode::INSERT && !array_key_exists($url, $LAST_ENTRIES)) {
    return;
  }
  
  // update last entries (as permalink URLs pointing to formatted times)
  if ($updateMode == UpdateMode::INSERT) {
    $LAST_ENTRIES = array($url => date("r")) + $LAST_ENTRIES;
  }
  else if ($updateMode == UpdateMode::DELETE) {
    unset($LAST_ENTRIES[$url]);
  }
  
  // HACK (not very elegant) to get absolute path to blog
  $protocol = isset($_SERVER['HTTPS']) ? "https" : "http";
  $serverAddress = $protocol."://".$_SERVER['HTTP_HOST'];
  $absolutePathToBlog = $serverAddress.$_SERVER['PHP_SELF']; // unpruned
  $levelsCount = substr_count($pathToBlog, ".."); // since pathToBlog are expressed as ../../ [...]
  for ($L=0; $L<$levelsCount; $L++) {
    $absolutePathToBlog = substr($absolutePathToBlog, 0, strrpos($absolutePathToBlog, "/"));
  }
  // finally, get rid of folder name [admin]
  $absolutePathToBlog = substr($absolutePathToBlog, 0, strrpos($absolutePathToBlog, "/"))."/";
  
  $titleArray = file($pathToBlog."elems/title");
  $blogTitle = implode("", $titleArray);
  
  $subtitleArray = file($pathToBlog."elems/subtitle");
  $blogSubtitle = implode("", $subtitleArray);
  
  $blogCategories = array();
  $categorie = strtok($blogSubtitle, ",");
  while ($categorie !== FALSE) {
    $categorie = trim($categorie);
    $blogCategories[] = $categorie;
    $categorie = strtok(",");
  }
  
  
  // write header
  
  $rssFile = fopen($pathToBlog."rss_feed/rss.xml", 'w');
  fwrite($rssFile, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
  fwrite($rssFile, "<rss version=\"2.0\" xmlns:atom=\"http://www.w3.org/2005/Atom\">\n");
  fwrite($rssFile, "<channel>\n");
  fwrite($rssFile, "<title>".$blogTitle."</title>\n");
  fwrite($rssFile, "<link>".$absolutePathToBlog."</link>\n");
  fwrite($rssFile, "<description>".$blogSubtitle."</description>\n");
  fwrite($rssFile, "<lastBuildDate>".date('r')."</lastBuildDate>\n");
  fwrite($rssFile, "<generator>siyen v0.9 - https://github.com/benjamin-auder/siyen</generator>\n");
  fwrite($rssFile, "<atom:link href=\"".$absolutePathToBlog."\" rel=\"self\" type=\"application/rss+xml\"/>\n");
  foreach ($blogCategories as $category) {
    fwrite($rssFile, "<category>".$category."</category>\n");
  }
  

  // write items + re-write LAST_ENTRIES array
  
  $lastEntriesFile = fopen($pathToBlog."rss_feed/last_entries.php", 'w');
  fwrite($lastEntriesFile, "<?php \$LAST_ENTRIES = array(\n");
  $countItems = 0;
  $firstLine = TRUE;
  foreach ($LAST_ENTRIES as $postUrl => $formattedDate) {
  
    fwrite($lastEntriesFile, "  ");
    if (!$firstLine) fwrite($lastEntriesFile, ",");
    $firstLine = FALSE;
    fwrite($lastEntriesFile, "'$postUrl' => '$formattedDate'\n");
    
    $post = Post::fromFile($pathToBlog.DataUrlParams::fromUrl($postUrl, ObjType::POST)->toUrl(UrlType::FILE));
    $permalink = $absolutePathToBlog.$postUrl;
    
    $postCategories = array();
    $categorie = strtok($post->getKeywords(), ",");
    while ($categorie !== FALSE) {
      $categorie = trim($categorie);
      $postCategories[] = $categorie;
      $categorie = strtok(",");
    }
    
    // get first and last name from user infos
    // TODO: also add these infos to posts ? or delete emails from posts ? ...
    $firstName = ".";
    $lastName = ".";
    $dirUsers = opendir($pathToBlog."admin/users_infos/");
    while (($file = readdir($dirUsers)) !== FALSE) {
      if ($file[0] == '.') continue;
      $user = User::fromFile($pathToBlog."admin/users_infos/".$file);
      if ($user->getAuthor() == $post->getAuthor()) {
        if (strlen($user->getFirstName()) > 0) {
          $firstName = $user->getFirstName();
        }
        if (strlen($user->getLastName()) > 0) {
          $lastName = $user->getLastName();
        }
        break;
      }
    }
    
    fwrite($rssFile, "<item>\n");
    fwrite($rssFile, "<guid isPermaLink=\"false\">".$postUrl."</guid>\n");
    fwrite($rssFile, "<title>".$post->getTitle()."</title>\n");
    fwrite($rssFile, "<link>".$permalink."</link>\n");
    fwrite($rssFile, "<description> <![CDATA[\n");
    fwrite($rssFile, $post->getText()."\n");
    fwrite($rssFile, "]]> </description>\n");
    fwrite($rssFile, "<author>".$post->getEmail()." (".$firstName." ".$lastName.")</author>\n");
    fwrite($rssFile, "<pubDate>".$formattedDate."</pubDate>\n");
    foreach ($postCategories as $category) {
      fwrite($rssFile, "<category>".$category."</category>\n");
    }
    fwrite($rssFile, "</item>\n");
    $countItems++;
    
    if ($countItems >= MAX_RSS_ITEMS) break; // if insertion + full buffer, skip the last item
  }
  
  // finish XML document
  fwrite($rssFile, "</channel></rss>\n");
  fclose($rssFile);
  
  // and last entries:
  fwrite($lastEntriesFile, "); ?>\n");
  fclose($lastEntriesFile);
}

class UpdateMode {
  const INSERT = 0;
  const EDIT = 1;
  const DELETE = 2;
}

?>
