<?php

/// Posts by time. Usage: search/index

class ScanTime {
  
  private static function isValidYear($year, $endYear) {
    return $year <= $endYear;
  }
  // assumption: year is valid
  private static function isValidMonth($year, $month, $endYear, $endMonth) {
    return $year < $endYear || ($year == $endYear && $month <= $endMonth);
  }
  // assumption: year and month are valid
  private static function isValidDay($year, $month, $day, $endYear, $endMonth, $endDay) {
    return $year < $endYear || ($year == $endYear && ($month < $endMonth || ($month == $endMonth && $day <= $endDay)));
  }
  
  // TODO: extend to searches like (blabla + blabla) | blabla + "blablabla..." ?
  // maybe, at least, allow exact strings inside 'AND' or 'OR' search.
  private static function countPatternsInText($pattern, $text) {
    
    if ($pattern[0] == '"') {
      // exact string matching
      $pattern = substr($pattern, 1, strlen($pattern) - 2); // remove '"' characters
      return substr_count($text, $pattern);
    }
    
    if (preg_match("/[ \n\t]\+[ \n\t]/", $pattern)) {
      // 'AND' search (terms separated by '+')
      $curIndex = 0;
      $res = PHP_INT_MAX;
      while ($curIndex < strlen($pattern)) {
        $end = $curIndex;
        while ($end < strlen($pattern) && $pattern[$end] != ' ' && $pattern[$end] != '\n' && $pattern[$end] != '\t') $end++;
        $word = substr($pattern, $curIndex, $end - $curIndex);
        if (strlen($word) == 0) break;
        $wordCount = substr_count($text, $word);
        if ($wordCount < $res) $res = $wordCount;
        // position to next start if possible:
        $curIndex = $end;
        while ($curIndex < strlen($pattern) && 
          ($pattern[$curIndex] == ' ' || $pattern[$curIndex] == '\n' || $pattern[$curIndex] == '\t' || $pattern[$curIndex] == '+'))
        {
          $curIndex++;
        }
      }
      return $res;
    }
    
    // 'OR' search
    $curIndex = 0;
    $res = 0;
    while ($curIndex < strlen($pattern)) {
      $end = $curIndex;
      while ($end < strlen($pattern) && $pattern[$end] != ' ' && $pattern[$end] != '\n' && $pattern[$end] != '\t') $end++;
      $word = substr($pattern, $curIndex, $end - $curIndex);
      if (strlen($word) == 0) break;
      $res += substr_count($text, $word);
      // position to next start if possible:
      $curIndex = $end;
      while ($curIndex < strlen($pattern) && 
        ($pattern[$curIndex] == ' ' || $pattern[$curIndex] == '\n' || $pattern[$curIndex] == '\t'))
      {
        $curIndex++;
      }
    }
    return $res;
  }
  
  // NOTE: $pathToBlog = "../"
  private static function countPatternsInThread($url, $pattern, $searchInPosts, $searchInTitles, $searchInKeywords, $searchInComments) {
    
    // TODO: notify if we found in text, keywords, or...
    
    $post = Post::fromFile("../".DataUrlParams::fromUrl($url, ObjType::POST)->toUrl(UrlType::FILE));
    $countPatterns = 0;
    if ($searchInPosts) {
      $countPatterns += ScanTime::countPatternsInText($pattern, $post->getText());
    }
    if ($searchInTitles) {
      $countPatterns += ScanTime::countPatternsInText($pattern, $post->getTitle());
    }
    if ($searchInKeywords) {
      $countPatterns += ScanTime::countPatternsInText($pattern, $post->getKeywords());
    }
    
    if ($searchInComments) {
      $dirComments = "../".DataUrlParams::fromUrl($url, ObjType::COMMENT)->toUrl(UrlType::FOLDER);
      $dir = opendir($dirComments);
      while (($file = readdir($dir)) !== FALSE) {
        $comment = Comment::fromFile($dirComments.$file);
        $countPatterns += ScanTime::countPatternsInText($pattern, $comment->getText());
      }
      closedir($dir);
    }
    
    return $countPatterns;
  }

  // Iterate through data/ to retrieve $maxPosts posts; HACK: $pathToBlog = "../"
  public static function getThreadsRange($pattern, $endTime, $maxResults, $searchInPosts, $searchInTitles, $searchInKeywords, $searchInComments) {
  
    $res = array();
    
    $lastYear = 3000;
    $lastMonth = 1;
    $lastDay = 1;
    $lastId = 0;
    
    // clean search pattern
    $pattern = trim($pattern, " \n\t");
    
    $basePath = "../data";
    
    $endYear = intval(substr($endTime, 0, 4));
    $endMonth = intval(substr($endTime, 5, 2));
    $endDay = intval(substr($endTime, 8, 2));
    $endId = intval(substr($endTime, 11, strlen($endTime) - 11));
    // endId == technical parameter, if multiple posts on same day and ask for continuation
    
    // step 1 == iterate through first level (year ; and sort)
    $dir = opendir($basePath);
    $fdir1 = array();
    $i = 0;
    while (($file = readdir($dir)) !== FALSE) {
      if ($file[0] != '.' && ScanTime::isValidYear(intval($file), $endYear)) {
        $fdir1[$i++] = $file;
      }
    }
    closedir($dir);
    rsort($fdir1, SORT_NUMERIC);
    $countDir1 = count($fdir1);
    
    $countResults = 0;
    $i = 0;
    while ($i < $countDir1 && $countResults < $maxResults) {
        
      // step 2 == iterate through second level (month ; and sort)
      $dir = opendir($basePath."/".$fdir1[$i]);
      $year = intval($fdir1[$i]);
      $fdir2 = array();
      $j = 0;
      while (($file = readdir($dir)) !== FALSE) { 
        if ($file[0] != '.' && ScanTime::isValidMonth($year, intval($file), $endYear, $endMonth)) {
          $fdir2[$j++] = $file;
        }
      }
      closedir($dir);
      rsort($fdir2, SORT_NUMERIC);
      $countDir2 = count($fdir2);
      
      $j = 0;
      while ($j < $countDir2 && $countResults < $maxResults) {
          
        // step 3 == iterate through third level (day ; and sort)
        $dir = opendir($basePath."/".$fdir1[$i]."/".$fdir2[$j]);
        $month = intval($fdir2[$j]);
        $fdir3 = array();
        $k = 0;
        while (($file = readdir($dir)) !== FALSE) {
          if ($file[0] != '.' && ScanTime::isValidDay($year, $month, intval($file), $endYear, $endMonth, $endDay)) {
            $fdir3[$k++] = $file;
          }
        }
        closedir($dir);
        rsort($fdir3, SORT_NUMERIC);
        $countDir3 = count($fdir3);
        
        $k = 0;
        while ($k < $countDir3 && $countResults < $maxResults) {
          
          // final step == iterate through thread folders
          $dir = opendir($basePath."/".$fdir1[$i]."/".$fdir2[$j]."/".$fdir3[$k]);
          $day = intval($fdir3[$k]);
          $threadFolders = array();
          $m = 0;
          while (($threadDir = readdir($dir)) !== FALSE) {
            if ($threadDir[0] == '.') continue;
            // we know that the date is valid; what about ID ?
            $tid = intval($threadDir);
            // check if ID is <= endId startID (continuation)
            if ($tid > $endId && $year==$endYear && $month==$endMonth && $day==$endDay) {
              continue;
            }
            $threadFolders[$m++] = $threadDir;
          }
          closedir($dir);
          rsort($threadFolders, SORT_NUMERIC);
          $countFolders = count($threadFolders);

          $m = 0;
          while ($m < $countFolders && $countResults < $maxResults) {
            
            // got a candidate: analyse its content for searched pattern
            $pathToThread = $basePath."/".$fdir1[$i]."/".$fdir2[$j]."/".$fdir3[$k]."/".$threadFolders[$m];
            $countPatterns = ScanTime::countPatternsInThread($pathToThread, $pattern, 
                    $searchInPosts, $searchInTitles, $searchInKeywords, $searchInComments);
            
            if ($countPatterns > 0) {
              $res[$pathToThread] = $countPatterns;
              $lastYear = $year;
              $lastMonth = $month;
              $lastDay = $day;
              $lastId = intval($threadFolders[$m]);
              $countResults++;
            }
            
            $m++;
          }
          $k++;
        }
        $j++;
      }
      $i++;
    }

    // adjust last date (inclusive !)
    if ($lastId > 0) $nextEndDate = $lastYear."-".$lastMonth."-".$lastDay.":".($lastId - 1);
    else if ($lastDay > 1) $nextEndDate = $lastYear."-".$lastMonth."-".($lastDay - 1).":".PHP_INT_MAX;
    else if ($lastMonth > 1) $nextEndDate = $lastYear."-".($lastMonth - 1)."-".ScanTime::lastDayInMonth($lastMonth-1).":".PHP_INT_MAX;
    else $nextEndDate = ($lastYear - 1)."-12-31:".PHP_INT_MAX;
    
    return array("search" => $res, "enddate" => $nextEndDate);
  }

  private static function lastDayInMonth($year, $month) {
    if ($month == 1 || $month == 3 || $month == 5 || $month == 7 || $month == 8 || $month == 10 || $month == 12) return 31;
    if ($month == 4 || $month == 6 || $month == 9 || $month == 11) return 30;
    return $year % 100 == 0 && $year % 400 != 0 ? 29 : 28; // february...
  }
  
  // Iterate through data/ to check for more within specified time-range;
  // note: still need to sort, because ID (inclusive) has to be checked as well on last level.
  // note: however, sorting is not required after first loop
  public static function checkMoreThreads($pattern, $nextEndTime, $searchInPosts, $searchInTitles, $searchInKeywords, $searchInComments) {
  
    $basePath = "../data";
    
    $endYear = intval(substr($nextEndTime, 0, 4));
    $endMonth = intval(substr($nextEndTime, 5, 2));
    $endDay = intval(substr($nextEndTime, 8, 2));
    $endId = intval(substr($nextEndTime, 11, strlen($nextEndTime) - 11));
    
    // step 1 == iterate through first level (year ; and sort)
    $dir = opendir($basePath);
    $fdir1 = array();
    $i = 0;
    while (($file = readdir($dir)) !== FALSE) {
      if ($file[0] != '.' && ScanTime::isValidYear(intval($file), $endYear)) {
        $fdir1[$i++] = $file;
      }
    }
    closedir($dir);
    rsort($fdir1, SORT_NUMERIC);
    $countDir1 = count($fdir1);
    
    $i = 0;
    while ($i < $countDir1) {
      
      // step 2 == iterate through second level (month ; and sort)
      $dir = opendir($basePath."/".$fdir1[$i]);
      $year = intval($fdir1[$i]);
      $fdir2 = array();
      $j = 0;
      while (($file = readdir($dir)) !== FALSE) { 
        if ($file[0] != '.' && ScanTime::isValidMonth($year, intval($file), $endYear, $endMonth)) {
          $fdir2[$j++] = $file;
        }
      }
      closedir($dir);
      rsort($fdir2, SORT_NUMERIC);
      $countDir2 = count($fdir2);
      
      $j = 0;
      while ($j < $countDir2) {
          
        // step 3 == iterate through third level (day ; and sort)
        $dir = opendir($basePath."/".$fdir1[$i]."/".$fdir2[$j]);
        $month = intval($fdir2[$j]);
        $fdir3 = array();
        $k = 0;
        while (($file = readdir($dir)) !== FALSE) {
          if ($file[0] != '.' && ScanTime::isValidDay($year, $month, intval($file), $endYear, $endMonth, $endDay)) {
            $fdir3[$k++] = $file;
          }
        }
        closedir($dir);
        rsort($fdir3, SORT_NUMERIC);
        $countDir3 = count($fdir3);
      
        $k = 0;
        while ($k < $countDir3) {
          
          // final step == iterate through post folders
          $dir = opendir($basePath."/".$fdir1[$i]."/".$fdir2[$j]."/".$fdir3[$k]);
          $day = intval($fdir3[$k]);
          while (($threadDir = readdir($dir)) !== FALSE) {
            
            if ($threadDir[0] == '.') continue;
            
            $pid = intval($threadDir);
            // check if ID is <= endId startID (continuation)
            if ($pid  > $endId && $year==$endYear && $month==$endMonth && $day==$endDay) {
              continue;
            }
            
            // got a candidate: analyse its content for searched pattern
            $countPatterns = ScanTime::countPatternsInThread(
                    $basePath."/".$fdir1[$i]."/".$fdir2[$j]."/".$fdir3[$k]."/".$threadDir,
                    $pattern, $searchInPosts, $searchInTitles, $searchInKeywords, $searchInComments);
            if ($countPatterns > 0) return TRUE; // OK, no need to explore further right now
          }
          closedir($dir);
          
          $k++;
        }
        $j++;
      }
      $i++;
    }
  
    return FALSE; // no match within scanned items
  }
}

?>
