<?php

/// Object encapsulating information of a comment

class Comment {

  private $id; // required to distinguish comments on blog index or permalink
  private $url; // permalink url with this comment
  private $author;
  private $authorUrl; // optional website of the author
  private $ctime;
  private $footprint;
  private $text;
  private $withCode;
  private $withMath;
  
  public function getId() {
    return $this->id;
  }
  public function getUrl() {
    return $this->url;
  }
  public function getAuthor() {
    return $this->author;
  }
  public function getAuthorUrl() {
    return $this->authorUrl;
  }
  public function getTime() {
    return $this->ctime;
  }
  public function getFootprint() {
    return $this->footprint;
  }
  public function getText() {
    return $this->text;
  }
  
  public function hasCode() {
    return $this->withCode;
  }
  public function hasMath() {
    return $this->withMath;
  }
  
  public function __construct($id, $url, $author, $authorUrl, $ctime, $footprint, $text, $withCode=FALSE, $withMath=FALSE) {
  
    $this->id = $id;
    $this->url = $url;
    $this->author = $author;
    $this->authorUrl = $authorUrl;
    $this->ctime = $ctime;
    $this->footprint = $footprint;
    $this->text = $text;
    $this->withCode = $withCode;
    $this->withMath = $withMath;
  }

  public function toFile($file) {
  
    $writer = fopen($file, 'w');
  
    fwrite($writer, $this->id."\n");
    fwrite($writer, $this->url."\n");
    fwrite($writer, $this->author."\n");
    fwrite($writer, $this->authorUrl."\n");
    fwrite($writer, $this->ctime."\n");
    fwrite($writer, $this->footprint."\n");
    fwrite($writer, ($this->withCode ? "TRUE" : "FALSE")."\n");
    fwrite($writer, ($this->withMath ? "TRUE" : "FALSE")."\n");
    fwrite($writer, $this->text);
    
    fclose($writer);
  }
  
  public static function fromFile($file) {
  
    $reader = fopen($file, 'r');
    
    // warning: an endline character is added
    // at the end of each non-terminal line.
    $id = substr(fgets($reader), 0, -1);
    $url = substr(fgets($reader), 0, -1);
    $author = substr(fgets($reader), 0, -1);
    $authorUrl = substr(fgets($reader), 0, -1);
    $ctime = substr(fgets($reader), 0, -1);
    $footprint = substr(fgets($reader), 0, -1);
    $withCode = substr(fgets($reader), 0, -1) == "TRUE";
    $withMath = substr(fgets($reader), 0, -1) == "TRUE";
    $text = "";
    while ( !feof($reader) ) {
      $text .= fgets($reader);
    }
    
    fclose($reader);
    
    return new Comment($id, $url, $author, $authorUrl, $ctime, $footprint, $text, $withCode, $withMath);
  }

  public function toHtml() {
    
    $res = "<div id='comment_".$this->id."' class='comment'>\n";
    
    $res .= "<p><span class='commentAuthor'>";
    if ($this->authorUrl != "") {
      $res .= "<a href='".$this->authorUrl."'>".$this->author."</a>";
    }
    else $res .= $this->author;
    $res .= "</span>";
    $res .= "<span class='commentTime'> on ".$this->ctime."</span></p>\n";

    $res .= "<div class='commentText'>".$this->text."</div></p>\n"; // main text
    
    $res .= "<p><hr class='hrText'/></p>\n";
    
    $res .= "</div>\n";
    
    return $res;
  }
  
  // from http://anujpathania.blogspot.com/2008/09/php-ip-logging-script.html
  // TODO: the IP adress is probably enough
  public static function getConnexionInfos() {
    
    $ip = $_SERVER['REMOTE_ADDR'];
    $pagina = $_SERVER['REQUEST_URI'];
    $datum = date("d-m-y / H:i:s");
    $invoegen = $datum . " - " . $ip . " - " . $pagina;
    return $invoegen;
  }
}

?>
