<?php

/// Class to abstract informations about someone who follow some thread

class Follower {
  
  private $id;
  private $email;
  private $permalink; // relative permalink, without "./"
  private $salt; // technical parameter to avoid unauthorized unsubscriptions
  
  public function getId() {
    return $this->id;
  }
  
  public function getEmail() {
    return $this->email;
  }
  public function getPermalink() {
    return $this->permalink;
  }
  public function getSalt() {
    return $this->salt;
  }
  
  public function __construct($id, $email, $permalink, $salt) {
    
    $this->id = $id;
    $this->email = $email;
    $this->permalink = $permalink;
    $this->salt = $salt;
  }
  
  public function notify($postTitle, $commentAuthor, $authorUrl, $commentTime, $commentText, $pathToBlog) {
    
    // HACK (not very elegant) to get *absolute* path to blog
    $protocol = isset($_SERVER['HTTPS']) ? "https" : "http";
    $serverAddress = $protocol."://".$_SERVER['HTTP_HOST'];
    $absolutePathToBlog = $serverAddress.$_SERVER['PHP_SELF']; // unpruned
    $levelsCount = substr_count($pathToBlog, ".."); // since pathToBlog are expressed as ../../ [...]
    for ($L=0; $L<$levelsCount; $L++) {
      $absolutePathToBlog = substr($absolutePathToBlog, 0, strrpos($absolutePathToBlog, "/"));
    }
    // finally, get rid of folder name [admin]
    $absolutePathToBlog = substr($absolutePathToBlog, 0, strrpos($absolutePathToBlog, "/"))."/";
    
    $permalink = $absolutePathToBlog.$this->permalink;
    $unsubscribeLink = $absolutePathToBlog."follow_thread/unsubscribe.php?id=".$this->id."&salt=".$this->salt."&email=".$this->email."&url=".$this->url;
    
    // send notification by email (text + HTML)
    // From http://www.siteduzero.com/tutoriel-3-35146-e-mail-envoyer-un-e-mail-en-php.html [in french]

    $boundary = "-----=".md5(rand());
    $serverName = $_SERVER['HTTP_HOST'];
    $leftPartServerName = strstr($serverName, ".", TRUE);

    $header = "From: \"$serverName\"<$leftPartServerName@$serverName>\n";
    $header.= "Reply-to: \"$serverName\" <noreply@$serverName>\n";
    $header.= "MIME-Version: 1.0\n";
    $header.= "Content-Type: text/plain;\n";
    $header .= "boundary=\"$boundary\"\n";

    $subject = "[siyen blog] '".$postTitle."' got a new comment.";

    $messageHeader_text = "Comment added to '".$postTitle."',\n";
    $messageHeader_text .= "by ".$commentAuthor." on ".$commentTime."\n";
    $messageHeader_text .= "-----------------------------------\n\n";
    
    $messageHeader_html = "Comment added to <a href='".$permalink."'>".$postTitle."</a>,<br/>\n";
    $messageHeader_html .= "by <a href='".$authorUrl."'>".$commentAuthor."</a> on ".$commentTime."\n";
    $messageHeader_html .= "<hr/>\n\n";
    
    $messageFooter_text = "To stop receiving updates, click the following link:\n";
    $messageFooter_text .= $unsubscribeLink;
    
    $messageFooter_html = "To stop receiving updates, click the following link:\n";
    $messageFooter_html .= "<a href='".$unsubscribeLink."'>Unsubscribe from notifications</a>";
    
    $message_txt = $messageHeader_text.strip_tags($commentText)."\n\n".$messageFooter_text;
    // TODO: transform 'custom' HTML (CSS...) into compatible HTML.
    $message_html = $messageHeader_html.$commentText."\n\n".$messageFooter_html;

    $message = "\n--".$boundary."\n";
    $message .= "Content-Type: text/plain; charset=\"ISO-8859-1\"\n";
    $message .= "Content-Transfer-Encoding: 8bit\n";
    $message .= "\n".$message_txt."\n";
    
    $message.= $passage_ligne."--".$boundary.$passage_ligne;

    $message.= "Content-Type: text/html; charset=\"ISO-8859-1\"\n";
    $message.= "Content-Transfer-Encoding: 8bit\n";
    $message.= "\n".$message_html."\n";
    $message.= "\n--".$boundary."--\n";
    $message.= "\n--".$boundary."--\n";
    
    // output the result of 'mail()' function, to remove follower in case of failure (?!)
    return mail($this->email, $subject, $message, $header);
  }
  
  public function toFile($file) {
    
    $writer = fopen($file, 'w');
  
    fwrite($writer, $this->id."\n");
    fwrite($writer, $this->email."\n");
    fwrite($writer, $this->permalink."\n");
    fwrite($writer, $this->salt."\n");
    
    fclose($writer);
  }
  
  public static function fromFile($file) {
    
    $reader = fopen($file, 'r');
    
    // warning: an endline character is added
    // at the end of each non-terminal line.
    $fid = substr(fgets($reader), 0, -1);
    $email = substr(fgets($reader), 0, -1);
    $permalink = substr(fgets($reader), 0, -1);
    $salt = substr(fgets($reader), 0, -1);
    
    fclose($reader);
    
    return new Follower($fid, $email, $permalink, $salt);
  }
}

?>
