<?php

/// Object encapsulating information of a post

class Post {

  private $pid; // post identifier (rank in day)
  private $author;
  private $email;
  private $ptime;
  private $lastedit;
  private $keywords;
  private $title;
  private $permalink;
  private $text;
  private $withCode;
  private $withMath;
  
  public function getId() {
    return $this->pid;
  }
  
  public function getAuthor() {
    return $this->author;
  }
  public function getEmail() {
    return $this->email;
  }
  public function getTime() {
    return $this->ptime;
  }
  public function getLastedit() {
    return $this->lastedit;
  }
  public function getKeywords() {
    return $this->keywords;
  }
  public function getTitle() {
    return $this->title;
  }
  public function getPermalink() {
    return $this->permalink;
  }
  public function getText() {
    return $this->text;
  }
  
  public function hasCode() {
    return $this->withCode;
  }
  public function hasMath() {
    return $this->withMath;
  }
  
  public function __construct($pid, $author, $email, $ptime, $lastedit, $keywords, 
          $title, $permalink, $text, $withCode=FALSE, $withMath=FALSE) {
  
    $this->pid = $pid;
    $this->author = $author;
    $this->email = $email;
    $this->ptime = $ptime;
    $this->lastedit = $lastedit;
    $this->keywords = $keywords;
    $this->title = $title;
    $this->permalink = $permalink;
    $this->text = $text;
    $this->withCode = $withCode;
    $this->withMath = $withMath;
  }

  public function toFile($file) {
  
    $writer = fopen($file, 'w');
  
    fwrite($writer, $this->pid."\n");
    fwrite($writer, $this->author."\n");
    fwrite($writer, $this->email."\n");
    fwrite($writer, $this->ptime."\n");
    fwrite($writer, $this->lastedit."\n");
    fwrite($writer, $this->keywords."\n");
    fwrite($writer, $this->title."\n");
    fwrite($writer, $this->permalink."\n");
    fwrite($writer, ($this->withCode ? "TRUE" : "FALSE")."\n");
    fwrite($writer, ($this->withMath ? "TRUE" : "FALSE")."\n");
    fwrite($writer, $this->text);
    
    fclose($writer);
  }
  
  public static function fromFile($file) {
  
    $reader = fopen($file, 'r');
    // warning: an endline character is added
    // at the end of each non-terminal line.
    $pid = intval(substr(fgets($reader), 0, -1));
    $author = substr(fgets($reader), 0, -1);
    $email = substr(fgets($reader), 0, -1);
    $ptime = substr(fgets($reader), 0, -1);
    $lastedit = substr(fgets($reader), 0, -1);
    $keywords = substr(fgets($reader), 0, -1);
    $title = substr(fgets($reader), 0, -1);
    $permalink = substr(fgets($reader), 0, -1);
    $withCode = substr(fgets($reader), 0, -1) == "TRUE";
    $withMath = substr(fgets($reader), 0, -1) == "TRUE";
    $text = "";
    while ( !feof($reader) ) {
      $text .= fgets($reader);
    }
    
    fclose($reader);
    
    return new Post($pid, $author, $email, $ptime, $lastedit, $keywords, $title, 
            $permalink, $text, $withCode, $withMath);
  }
  
  public function toHtml($countComments = -1, $countTrackbacks = -1) {

    $res = "<div class='post' id='post_".$this->pid."'>\n";
    
    $res .= "<h1><a href='".$this->permalink."'>".$this->title."</a></h1>\n";
    
    $res .= "<p><span class='postAuthor'>by <a href='mailto:".$this->email."'>".$this->author."</a></span>";
    $res .= "<span class='postTime'> on ".$this->ptime."</span></p>\n";
    
    $res .= "<div class='postText'>".$this->text."</div>\n"; // main text
    
    $res .= "<p><span class='postFooter'>\n";
    
    if ($countComments >= 0) {
      // means "index.php", where comments are summed up
      $res .= "<a href='".$this->permalink."'>".$countComments." comments</a>";
      $res .= "<a href='".$this->permalink."'> || ".$countTrackbacks." trackbacks</a>\n";
    }
    
    if ($this->lastedit != "") $res .= " || last edit on ".$this->lastedit."\n";
    $res .= "</span></p>\n";
    
    $res .= "<p><hr class='hrText'/></p>\n";
    
    $res .= "</div>";
    
    return $res;
  }
}

?>
