<?php

/// PHP Class to handle TrackBacks (send/ping, receive, retrieve, detect, seed, etc...)
/// Initially adapted from trackback_cls.php of Ran Aroussi at http://phptrackback.sourceforge.net/

// thanks also to: http://www.movabletype.org/documentation/trackback_manual.html
// very few resources !

class Trackback {
  
  private $tid; // trackBack ID, for the ones which get logged
  private $url;
  private $title;
  private $blog_name;
  private $excerpt;
  
  // Avoid overflow : no more than 10 potential trackback links
  const MAX_TB_LINKS = 10;
  
  public function getId() {
    return $this->tid;
  }
  public function getUrl() {
    return $this->url;
  }
  public function getTitle() {
    return $this->title;
  }
  public function getBlog_Name() {
    return $this->blog_name;
  }
  public function getExcerpt() {
    return $this->excerpt;
  }
  
  public function __construct($tid, $url, $title, $blog_name = "", $excerpt = "") {
    
    $this->tid = $tid;
    $this->url = $url;
    $this->title = $title;
    $this->blog_name = $blog_name;
    $this->excerpt = $excerpt;
  }
  
  public function toFile($file) {
  
    $writer = fopen($file, 'w');
  
    fwrite($writer, $this->tid."\n");
    fwrite($writer, $this->url."\n");
    fwrite($writer, $this->title."\n");
    fwrite($writer, $this->blog_name."\n");
    fwrite($writer, $this->excerpt."\n");
    
    fclose($writer);
  }
  
  public static function fromFile($file) {
    
    $reader = fopen($file, 'r');
    
    // warning: an endline character is added
    // at the end of each non-terminal line.
    $id = substr(fgets($reader), 0, -1);
    $url = substr(fgets($reader), 0, -1);
    $title = substr(fgets($reader), 0, -1);
    $blog_name = substr(fgets($reader), 0, -1);
    $excerpt = substr(fgets($reader), 0, -1);
    fclose($reader);
    
    return new Trackback($id, $url, $title, $blog_name, $excerpt);
  }
  
  public function toHtml() {

    $res = "<div class='trackback' id='trackback_".$this->tid."'>\n";
    
    $res .= "<h1><a href='".$this->url."'>".$this->title."</a></h1>\n";
    
    // HACK: no excerpt display for the moment
    if (!empty($this->blog_name)) {
      $res .= "From ".$this->blog_name."\n";
    }
    
    $res .= "</div>\n";
    
    return $res;
  }
  
  // TODO: UNTESTED !
  // TODO: rewrite in a simplified form, using AJAX (?!)
  public static function ping($tb) {
    
    // Parse the target
    $target = parse_url($tb);
    if ((isset($target["query"])) && ($target["query"] != "")) {
      $target["query"] = "?" . $target["query"];
    }
    else {
      $target["query"] = "";
    } 
    if ((isset($target["port"]) && !is_numeric($target["port"])) || (!isset($target["port"]))) {
      $target["port"] = 80;
    }
    
    // Open the socket
    $tb_sock = fsockopen($target["host"], $target["port"]);
    // Something didn't work out, return
    if (!is_resource($tb_sock)) {
      return "failure";
    }
    
    // Put together the things we want to send
    $tb_send = "url=" . rawurlencode($this->url) . "&title=" . rawurlencode($this->title) . "&blog_name=" . rawurlencode($this->blog_name);
    if (!empty($this->excerpt)) $tb_send .= "&excerpt=" . rawurlencode($this->excerpt);
    
    // Send the trackback
    fputs($tb_sock, "POST " . $target["path"] . $target["query"] . " HTTP/1.1\n");
    fputs($tb_sock, "Host: " . $target["host"] . "\n");
    fputs($tb_sock, "Content-type: application/x-www-form-urlencoded\n");
    fputs($tb_sock, "Content-length: " . strlen($tb_send) . "\n");
    fputs($tb_sock, "Connection: close\n\n");
    fputs($tb_sock, $tb_send);
    
    // Gather result
    $response = "";
    while (!feof($tb_sock)) {
      $response .= fgets($tb_sock, 128);
    }
    // Close socket
    fclose($tb_sock);
    
    // Did the trackback ping work
    return strpos($response, '<error>0</error>') >= 0
      ? "success"
      : "failure";
  }

  private static function getResponseError($message) {
    $response = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<response>\n<error>1</error>\n";
    $response .= "<message>$message</message>\n</response>";
    return $response;
  }

  // Receive a trackback request (someone has linked to our blog)
  // TODO: UNTESTED !
  public static function receive($my_url) {
    
    // Gather $_POST information
    if (!isset($_POST['url']) || !isset($_POST['title'])) {
      return Trackback::getResponseError("Wrong trackback format.");
    }
    
    $this->url = $_POST['url'];
    $this->title = $_POST['title'];
    
    $this->excerpt = isset($_POST['excerpt'])
      ? $_POST['excerpt']
      : "";
    $this->blog_name = isset($_POST['blog_name'])
      ? $_POST['blog_name']
      : "";
    
    
    // SPAM DETECTOR
    // (ideas from http://wordpress.org/extend/plugins/simple-trackback-validation/)
    
    // Check if IP associated to the URL corresponds to the trackback requester. If not, reject trackback.
    $fullUrl = $this->url;
    // need to isolate the domain name between 'http://' and '/blabla/blabla/index.php'
    $startIndex = strpos($fullUrl, "//") + 2;
    $endIndex = strpos($fullUrl, "/", $startIndex);
    if ($endIndex == FALSE) $endIndex = strlen($fullUrl) - 1;
    $domainName = substr($fullUrl, $startIndex, $endIndex - $startIndex + 1);
    if ($_SERVER['REMOTE_ADDR'] != gethostbyname($domainName)) {
      return Trackback::getResponseError("Spamming attempt detected.");
    }
    
    // Check if our post URL appears in the remote page; if not, we consider it's spam.
    $content = TrackBack::getRemoteText($url);
    if (strpos($content, $my_url) == 0) {
      return Trackback::getResponseError("Spamming attempt detected.");
    }
    
    
    // detect multiple trackback attempts from same url, to same my_url
    $trackbacksFolder = "./".DataUrlParams::fromUrl($my_url, ObjType::TRACKBACK)->toUrl(UrlType::FOLDER);
    $dir = opendir($trackbacksFolder);
    $tid = -1;
    // TODO: [identifier search] potential concurrent access issues. (same as for comments, and posts)
    while (($file = readdir($dir)) !== FALSE) {
      if ($file[0] == '.') continue;
      $tb = Trackback::fromFile($trackbacksFolder.$file);
      if ($tb->getUrl() == $my_url) {
        // already present: just do nothing
        return Trackback::getResponseError("Duplicate trackback to this URL.");
      }
      $tid = max($tid, intval($file));
    }
    $tid++;
    
    // Log in the file:
    $this->toFile("./".DataUrlParams::fromUrl($my_url, ObjType::TRACKBACK, $tid)->toUrl(UrlType::FILE));
    
    // Report that everything is OK
    return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<response>\n<error>0</error>\n</response>";
  }

  // Produces embedded RDF representing metadata about the entry,
  // allowing clients to auto-discover the TrackBack Ping URL.
  public static function rdfAutodiscover($title, $permalink, $trackback) {
    
    $rdf = "<!--\n";
    $rdf .= "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n";
    $rdf .= "	xmlns:dc=\"http://purl.org/dc/elements/1.1/\"\n";
    $rdf .= "	xmlns:trackback=\"http://madskills.com/public/xml/rss/module/trackback/\">\n";
    $rdf .= "<rdf:Description\n";
    $rdf .= "	rdf:about=\"".htmlspecialchars($permalink, ENT_QUOTES)."\"\n";
    $rdf .= "	dc:identifier=\"".htmlspecialchars($permalink, ENT_QUOTES)."\"\n";
    $rdf .= " dc:title=\"".htmlspecialchars($title, ENT_QUOTES)."\"\n";
    $rdf .= " trackback:ping=\"".htmlspecialchars($trackback, ENT_QUOTES)."\"\n";
    $rdf .= "</rdf:RDF>\n";
    $rdf .= "-->\n";
    return $rdf;
  }

  // Search text for links, and searches links for trackback URLs. [within REMOTE blog]
  // TODO: UNTESTED !
  public static function auto_discovery($link) {
    
    $text = TrackBack::getRemoteText($link);
    
    $tbLinks = array();
    $countTbLinks = 0;
    
    $startComment = strpos($text, "<!--");
    while ($startComment != FALSE && $countTbLinks < Trackback::MAX_TB_LINKS) {
      
      $startRDF = strpos($text, "<rdf:RDF", $startComment);
      while ($startRDF != FALSE && $countTbLinks < Trackback::MAX_TB_LINKS) {
        
        $startLink = strpos($text, "trackback:ping", $startRDF);
        while ($startLink != FALSE && $countTbLinks < Trackback::MAX_TB_LINKS) {
          
          $startIndex = $startLink + 16; // 14 characters, plus '=', plus quote (single or double)
          $quoteChar = $text[$startIndex - 1];
          $endIndex = strpos($text, $quoteChar, $startIndex);
          $tbLinks[] = substr($text, $startIndex, $endIndex - $startIndex);
          
          $countTbLinks++;
          
          $startLink = $endIndex;
        }
        
        $startRDF = strpos($text, "<rdf:RDF", $startLink);
      }
      
      $startComment = strpos($text, "<!--", $startRDF);
    }
    
    return $tbLinks;
  }

  // helper: get a remote text from URL
  // TODO: UNTESTED !
  private static function getRemoteText($url) {
    
    $fp = fopen($url, 'r');
    if ($fp === FALSE) {
      // cannot open file
      return NULL;
    }
    
    $content = "";
    while( !feof( $fp ) ) {
      $buffer = trim( fgets( $fp, 4096 ) );
      $content .= $buffer;
    }
    return $content;
  }
} 

?>
