<?php

/// Oject encapsulating information of a generic URL

class UrlParams {

  protected $id;
  protected $year;
  protected $month;
  protected $day;
  
  public function getId() {
    return $this->id;
  }
  public function getYear() {
    return $this->year;
  }
  public function getMonth() {
    return $this->month;
  }
  public function getDay() {
    return $this->day;
  }
  
  public function __construct($year, $month, $day, $id) {
    $this->year = $year;
    $this->month = $month;
    $this->day = $day;
    $this->id = $id;
  }
  
  protected static function isDigit($c) {
    return ($c >= '0' && $c <= '9');
  }
}


/// Object encapsulating information of a data (internal) URL

class ObjType {
  const POST = 0;
  const COMMENT = 1;
  const TRACKBACK = 2;
  const FOLLOWER = 3;
  const UPLOAD = 4;
}

class UrlType {
  // to distinguish comments folder from one comment text file:
  const FILE = 0;
  const FOLDER = 1;
}

class DataUrlParams extends UrlParams {

  private $objType; // a constant from the ObjType class above (0 to 3)
  private $index; // subindex, for (possible) comments and trackbacks
  
  public function __construct($year, $month, $day, $id, $objType, $index) {
    $this->year = $year;
    $this->month = $month;
    $this->day = $day;
    $this->id = $id;
    $this->objType = $objType;
    $this->index = $index;
  }

  // url end by year-month-day:ID
  // TODO: duplicated 3 times ! [see ArxivUrl and TrackbackUrl below]
  public static function fromUrl($url, $objType, $index = -1) {
  
    $end = strlen($url) - 1;
    while ( !UrlParams::isDigit($url[$end]) ) $end--;
    $begin = $end;
    $end++;
    while ( UrlParams::isDigit($url[$begin]) ) $begin--;
    $begin++;
    $id = intval(substr($url, $begin, $end - $begin));

    $begin = $begin - 11; // HACK: 8 date chars, 3 special signs
    $year = substr($url, $begin, 4);
    $month = substr($url, $begin + 5, 2);
    $day = substr($url, $begin + 8, 2);
    
    return new DataUrlParams($year, $month, $day, $id, $objType, $index);
  }
  
  public function toUrl($urlType) {
    
    $baseUrl = "data/".$this->year."/".$this->month."/".$this->day."/".$this->id."/";
    $suffix = "";
    switch ($this->objType) {
      case ObjType::POST:
        if ($urlType == UrlType::FILE) $suffix = "p";
        break;
      case ObjType::COMMENT:
        $suffix = "c/";
        if ($urlType == UrlType::FILE && $this->index >= 0) $suffix .= $this->index;
        break;
      case ObjType::TRACKBACK:
        $suffix = "t/";
        if ($urlType == UrlType::FILE && $this->index >= 0) $suffix .= $this->index;
        break;
      case ObjType::FOLLOWER:
        $suffix = "f/";
        if ($urlType == UrlType::FILE && $this->index >= 0) $suffix .= $this->index;
        break;
      case ObjType::UPLOAD:
        $suffix = "u/";
        break;
    }
    return $baseUrl.$suffix;
  }
}


/// Object encapsulating information of an arxiv (permalink) URL

class ArxivUrlParams extends UrlParams {
  
  // url is path_to_blog/arxiv.php?id=2xxx-xx-xx:ID
  public static function fromUrl($url) {
    
    $end = strlen($url) - 1;
    while ( !UrlParams::isDigit($url[$end]) ) $end--;
    $begin = $end;
    $end++;
    while ( UrlParams::isDigit($url[$begin]) ) $begin--;
    $begin++;
    $id = intval(substr($url, $begin, $end - $begin));

    $begin = $begin - 11; // HACK: 8 date chars, 3 special signs
    $year = substr($url, $begin, 4);
    $month = substr($url, $begin + 5, 2);
    $day = substr($url, $begin + 8, 2);
    
    return new ArxivUrlParams($year, $month, $day, $id);
  }
  
  public function toUrl() {
    return "arxiv.php?id=".$this->year."-".$this->month."-".$this->day.":".$this->id;
  }
}


/// Object encapsulating information of a trackback link URL (to receive pings)

class TrackbackUrlParams extends UrlParams {
  
  // url is path_to_blog/trackback.php?id=2xxx-xx-xx:ID
  public static function fromUrl($url) {
    
    $end = strlen($url) - 1;
    while ( !UrlParams::isDigit($url[$end]) ) $end--;
    $begin = $end;
    $end++;
    while ( UrlParams::isDigit($url[$begin]) ) $begin--;
    $begin++;
    $id = intval(substr($url, $begin, $end - $begin));

    $begin = $begin - 11; // HACK: 8 date chars, 3 special signs
    $year = substr($url, $begin, 4);
    $month = substr($url, $begin + 5, 2);
    $day = substr($url, $begin + 8, 2);
    
    return new TrackbackUrlParams($year, $month, $day, $id);
  }
  
  public function toUrl() {
    return "trackback.php?id=".$this->year."-".$this->month."-".$this->day.":".$this->id;
  }
}

?>
