<?php

/// Insert or remove keywords + keep a sort order

class KeywordsArray {
  
  private $allKeywords;
  
  public function getArray() {
    return $this->allKeywords;
  }
  
  public function __construct($keywordsArray) {
    $this->allKeywords = $keywordsArray;
  }
  
  public function insert($postKeywords) {
    
    $tokens = array();
    $token = strtok($postKeywords, ",");
    while ($token !== FALSE) {
      $tokens[] = trim($token, "\n\t ");
      $token = strtok(",");
    }
    $countTokens = count($tokens);
    
    for ($i=0; $i<$countTokens; $i++) {
      
      $subToken = trim(strtok($tokens[$i], "/"), "\n\t ");
      if (!array_key_exists($subToken, $this->allKeywords)) {
        $this->allKeywords[$subToken] = array("val" => 1, "suba" => array());
      }
      else $this->allKeywords[$subToken]["val"]++;
      $current = &$this->allKeywords[$subToken]["suba"];
      $subToken = strtok("/");
      
      while ($subToken !== FALSE) {
      	$subToken = trim($subToken, "\n\t ");
        if (!array_key_exists($subToken, $current)) {
          $current[$subToken] = array("val" => 1, "suba" => array());
        }
        else $current[$subToken]["val"]++;
        $current = &$current[$subToken]["suba"];
        $subToken = strtok("/");
      }
    }
  }
  
  public function remove($postKeywords) {
    
    $tokens = array();
    $token = strtok($postKeywords, ",");
    while ($token !== FALSE) {
      $tokens[] = trim($token, "\n\t ");
      $token = strtok(",");
    }
    $countTokens = count($tokens);
    
    for ($i=0; $i<$countTokens; $i++) {
      
      $subToken = trim(strtok($tokens[$i], "/"), "\n\t ");
      $this->allKeywords[$subToken]["val"]--;
      if ($this->allKeywords[$subToken]["val"] <= 0) {
        unset($this->allKeywords[$subToken]);
        continue; // no need to explore further: branch is cut
      }
      $current = &$this->allKeywords[$subToken]["suba"];
      $subToken = strtok("/");
      
      while ($subToken !== FALSE) {
      	$subToken = trim($subToken, "\n\t ");
        $current[$subToken]["val"]--;
        if ($current[$subToken]["val"] <= 0) {
          unset($current[$subToken]);
          break; // no need to explore further: branch is cut
        }
        $current = &$current[$subToken]["suba"];
        $subToken = strtok("/");
      }
    }
  }
  
  // keep a sort order
  public function rearrange($sortOrder) {
    KeywordsArray::rearrange_aux($this->allKeywords, $sortOrder);
  }
  
  private static function rearrange_aux(&$subArray, $sortOrder) {
    
    // termination
    if (count($subArray) == 0) return;
    
    // core logic
    switch ($sortOrder) {
      case SortOrder::ALPHABETIC:
        ksort($subArray);
        break;
      case SortOrder::NUMERIC:
        uasort($subArray, "cmp_on_value");
        break;
    }
    
    // recursive calls
    foreach ($subArray as $key => $val) {
      KeywordsArray::rearrange_aux($subArray[$key]["suba"], $sortOrder);
    }
  }
  
  // write keywords to file, using a depth-first tree exploration
  public function toFile($file) {
    
    $fileKeywords = fopen($file, 'w');
    fwrite($fileKeywords, "<?php \$RANKED_KEYWORDS = array(\n");
    $firstLine = TRUE;
    foreach ($this->allKeywords as $key => $vals) {
      KeywordsArray::toFile_aux($fileKeywords, $key, $vals, $firstLine);
      if ($firstLine) $firstLine = FALSE;
    }
    fwrite($fileKeywords, "); ?>\n");
    fclose($fileKeywords);
  }
  
  private static function toFile_aux($writer, $key, $subArray, $firstLine) {
    
    if (!$firstLine) fwrite($writer, ",");
    fwrite($writer, "'".$key."' => array('val' => ".$subArray["val"].", 'suba' => array(\n");
    $firstLine = TRUE;
    foreach ($subArray["suba"] as $key => $vals) {
      KeywordsArray::toFile_aux($writer, $key, $vals, $firstLine);
      if ($firstLine) $firstLine = FALSE;
    }
    fwrite($writer, "))\n");
  }
}

function cmp_on_value($a, $b) {
  if ($a == $b) return 0;
  return ($a < $b) ? -1 : 1;
}

class SortOrder {
  const ALPHABETIC = 0;
  const NUMERIC = 1;
}

?>
