<?php

// markups allowed: admin mode
class ParseAdmin {

  // List all <code> markup types
  private static $codeMarkups = array(
    // no numbering, no borders
    array("start" => "<code>", "end" => "</code>", "name" => "code"),
    // borders, no numbering
    array("start" => "<codeb>", "end" => "</codeb>", "name" => "codeb"),
    // numbering, no borders
    array("start" => "<codel>", "end" => "</codel>", "name" => "codel"),
    // borders and numbering
    array("start" => "<codebl>", "end" => "</codebl>", "name" => "codebl"));
  
  // post: all functionalities, trusted writers
  public static function transformText($text) {

    // replace simplified markups by CSS
    // The comments are useful to keep the untransform method light (see below).
    $text = str_replace("<i>", "<span class='italic'>", $text);
    $text = str_replace("</i>", "</span><!--i-->", $text);
    $text = str_replace("<b>", "<span class='bold'>", $text);
    $text = str_replace("</b>", "</span><!--b-->", $text);
    $text = str_replace("<u>", "<span class='uline'>", $text);
    $text = str_replace("</u>", "</span><!--u-->", $text);
    $text = str_replace("<s>", "<span class='strike'>", $text);
    $text = str_replace("</s>", "</span><!--s-->", $text);
    $text = str_replace("<huge>", "<span class='huge'>", $text);
    $text = str_replace("</huge>", "</span><!--huge-->", $text);
    $text = str_replace("<large>", "<span class='large'>", $text);
    $text = str_replace("</large>", "</span><!--large-->", $text);
    $text = str_replace("<small>", "<span class='small'>", $text);
    $text = str_replace("</small>", "</span><!--small-->", $text);
    $text = str_replace("<tiny>", "<span class='tiny'>", $text);
    $text = str_replace("</tiny>", "</span><!--tiny-->", $text);
    $text = str_replace("<box>", "<div class='boxed'>", $text);
    $text = str_replace("</box>", "</div><!--boxed-->", $text);
    $text = str_replace("<br>", "<br/>", $text);
    $text = str_replace("<hr>", "<hr/>", $text);
    
    // text alignment
    $text = str_replace("<left>", "<div class='alignLeft'>", $text);
    $text = str_replace("</left>", "</div><!--left-->", $text);
    $text = str_replace("<center>", "<div class='alignCenter'>", $text);
    $text = str_replace("</center>", "</div><!--center-->", $text);
    $text = str_replace("<right>", "<div class='alignRight'>", $text);
    $text = str_replace("</right>", "</div><!--right-->", $text);
    $text = str_replace("<indent>", "<div class='indent'>", $text);
    $text = str_replace("</indent>", "</div><!--indent-->", $text);
    
    // colors:
    $text = str_replace("<'#", "<span style='color:#", $text);
    $text = str_replace("</#>", "</span><!--#-->", $text);
    
    // maybe code: replace <code[bl]>Blabla</code[bl]> by 
    // <div class='code[bl]'> [Syntax_Highlighting] </div>
    foreach (ParseAdmin::$codeMarkups as $codeMarkup) {
      $lastOffset = 0;
      while (TRUE) {
        $codeStart = strpos($text, $codeMarkup["start"], $lastOffset);
        if ($codeStart === FALSE) break;
        $codeEnd = strpos($text, $codeMarkup["end"], $codeStart);
        $codeStart += strlen($codeMarkup["start"]); // shift to the code beginning
        $code = substr($text, $codeStart, $codeEnd - $codeStart);
        // NOTE: this is quite heavy
        $text = substr($text, 0, $codeStart - strlen($codeMarkup["start"])) // skip '<code*>'
          ."<div class='".$codeMarkup["name"]."'>"
          .SyntaxHighlighter::highlight($code)
          ."</div><!--".$codeMarkup["name"]."-->"
          .substr($text, $codeEnd + strlen($codeMarkup["end"]), // skip '</code*>'
                  strlen($text) - $codeEnd - strlen($codeMarkup["end"]));
        $lastOffset = $codeEnd;
        if ($lastOffset >= strlen($text)) break;
      }
    }

    // mathjax:
    // nothing to do: <math> ... </math> set as mathjax inline marker. (?!)
    
    return $text;
  }
  
  
  // List all inverse <code> markup types
  private static $invCodeMarkups = array(
    // no numbering, no borders
    array("start" => "<div class='code'>", "end" => "</div><!--code-->", "name" => "code"),
    // borders, no numbering
    array("start" => "<div class='codeb'>", "end" => "</div><!--codeb-->", "name" => "codeb"),
    // numbering, no borders
    array("start" => "<div class='codel'>", "end" => "</div><!--codel-->", "name" => "codel"),
    // borders and numbering
    array("start" => "<div class='codebl'>", "end" => "</div><!--codebl-->", "name" => "codebl"));
  
  // inverse transform for editing
  public static function untransformText($text) {

    // replace CSS by simplified markups
    $text = str_replace("<span class='italic'>", "<i>", $text);
    $text = str_replace("</span><!--i-->", "</i>", $text);
    $text = str_replace("<span class='bold'>", "<b>", $text);
    $text = str_replace("</span><!--b-->", "</b>", $text);
    $text = str_replace("<span class='uline'>", "<u>", $text);
    $text = str_replace("</span><!--u-->", "</u>", $text);
    $text = str_replace("<span class='strike'>", "<s>", $text);
    $text = str_replace("</span><!--s-->", "</s>", $text);
    $text = str_replace("<span class='huge'>", "<huge>", $text);
    $text = str_replace("</span><!--huge-->", "</huge>", $text);
    $text = str_replace("<span class='large'>", "<large>", $text);
    $text = str_replace("</span><!--large-->", "</large>", $text);
    $text = str_replace("<span class='small'>", "<small>", $text);
    $text = str_replace("</span><!--small-->", "</small>", $text);
    $text = str_replace("<span class='tiny'>", "<tiny>", $text);
    $text = str_replace("</span><!--tiny-->", "</tiny>", $text);
    $text = str_replace("<div class='boxed'>", "<box>", $text);
    $text = str_replace("</div><!--boxed-->", "</box>", $text);
    $text = str_replace("<br/>", "<br>", $text);
    $text = str_replace("<hr/>", "<hr>", $text);
    
    // text alignment
    $text = str_replace("<div class='alignLeft'>", "<left>", $text);
    $text = str_replace("</div><!--left-->", "</left>", $text);
    $text = str_replace("<div class='alignCenter'>", "<center>", $text);
    $text = str_replace("</div><!--center-->", "</center>", $text);
    $text = str_replace("<div class='alignRight'", "<right>", $text);
    $text = str_replace("</div><!--right-->", "</right>", $text);
    $text = str_replace("<div class='indent'", "<indent>", $text);
    $text = str_replace("</div><!--indent-->", "</indent>", $text);
    
    // colors:
    $text = str_replace("<span style='color:#", "<'#", $text);
    $text = str_replace("</span><!--#-->", "</#>", $text);
    
    // maybe code: replace <div class='code[bl]'> [Syntax_Highlighting] </div> by 
    // <code[bl]> strip_all_highlight_tags </code[bl]>
    foreach (ParseAdmin::$invCodeMarkups as $invCodeMarkup) {
      $lastOffset = 0;
      while (TRUE) {
        $codeStart = strpos($text, $invCodeMarkup["start"], $lastOffset);
        if ($codeStart === FALSE) break;
        $codeEnd = strpos($text, $invCodeMarkup["end"], $codeStart);
        $codeStart += strlen($invCodeMarkup["start"]); // shift to the code beginning
        $code = substr($text, $codeStart, $codeEnd - $codeStart);
        // NOTE: this is quite heavy
        $text = substr($text, 0, $codeStart - strlen($invCodeMarkup["start"])) // skip '<div class=*>'
          ."<".$invCodeMarkup["name"].">"
          .html_entity_decode(strip_tags($code))
          ."</".$invCodeMarkup["name"].">"
          .substr($text, $codeEnd + strlen($invCodeMarkup["end"]), // skip '</div><!--*-->'
                  strlen($text) - $codeEnd - strlen($invCodeMarkup["end"]));
        $lastOffset = $codeEnd;
        if ($lastOffset >= strlen($text)) break;
      }
    }

    // mathjax:
    // nothing to do: <math> ... </math> set as mathjax inline marker. (?!)
    
    return $text;
  }
}

?>
