<?php

// HTML markups not allowed: public mode (comments in posts, trackbacks...)
// NOTE: comments are immutable ==> no need for untransform

class ParsePublic {

  // List all <code> markup types
  private static $codeMarkups = array(
    // no numbering, no borders
    array("start" => "[code]", "end" => "[/code]", "name" => "code"),
    // borders, no numbering
    array("start" => "[codeb]", "end" => "[/codeb]", "name" => "codeb"),
    // numbering, no borders
    array("start" => "[codel]", "end" => "[/codel]", "name" => "codel"),
    // borders and numbering
    array("start" => "[codebl]", "end" => "[/codebl]", "name" => "codebl"));
  
  // comment: limited functionalities, untrusted writers.
  public static function transformComment($text) {
    
    // first, strip all HTML markups
    $text = strip_tags($text);
    
    // no javascript (in case of)
    $text = str_replace("<script", "", $text);
    $text = str_replace("</script", "", $text);
    
    // from here, same logic as in ParsePost (without code)
    
    // replace simplified markups by CSS
    // The comments are useful to keep the untransform method light (see below).
    $text = str_replace("[i]", "<span class='italic'>", $text);
    $text = str_replace("[/i]", "</span><!--i-->", $text);
    $text = str_replace("[b]", "<span class='bold'>", $text);
    $text = str_replace("[/b]", "</span><!--b-->", $text);
    $text = str_replace("[u]", "<span class='strike'>", $text);
    $text = str_replace("[/u]", "</span><!--s-->", $text);
    $text = str_replace("[u]", "<span class='uline'>", $text);
    $text = str_replace("[/u]", "</span><!--u-->", $text);
    $text = str_replace("[br]", "<br/>", $text);
    $text = str_replace("[hr]", "<hr/>", $text);
    
    // text alignment
    $text = str_replace("[left]", "<span class='alignLeft'>", $text);
    $text = str_replace("[/left]", "</span><!--left-->", $text);
    $text = str_replace("[center]", "<span class='center'>", $text);
    $text = str_replace("[/center]", "</span><!--center-->", $text);
    $text = str_replace("[right]", "<span class='alignRight'>", $text);
    $text = str_replace("[/right]", "</right><!--right-->", $text);
    
    // colors:
    $text = str_replace("['#", "<span style='color:#", $text);
    $text = str_replace("[/#]", "</span><!--#-->", $text);
    
    // maybe code: replace <code[bl]>Blabla</code[bl]> by 
    // <div class='code[bl]'> [Syntax_Highlighting] </div>
    foreach (ParsePublic::$codeMarkups as $codeMarkup) {
      $lastOffset = 0;
      while (TRUE) {
        $codeStart = strpos($text, $codeMarkup["start"], $lastOffset);
        if ($codeStart === FALSE) break;
        $codeEnd = strpos($text, $codeMarkup["end"], $codeStart);
        $codeStart += strlen($codeMarkup["start"]); // shift to the code beginning
        $code = substr($text, $codeStart, $codeEnd - $codeStart);
        // NOTE: this is quite heavy
        $text = substr($text, 0, $codeStart - strlen($codeMarkup["start"])) // skip '[code*]'
          ."<div class='".$codeMarkup["name"]."'>"
          .SyntaxHighlighter::highlight($code)
          ."</div><!--".$codeMarkup["name"]."-->"
          .substr($text, $codeEnd + strlen($codeMarkup["end"]), // skip '[/code*]'
                  strlen($text) - $codeEnd - strlen($codeMarkup["end"]));
        $lastOffset = $codeEnd;
        if ($lastOffset >= strlen($text)) break;
      }
    }

    // lists
    $text = str_replace("[li]", "<li>", $text);
    $text = str_replace("[/li]", "</li>", $text);
    $text = str_replace("[ul]", "<ul>", $text);
    $text = str_replace("[/ul]", "</ul>", $text);
    $text = str_replace("[ol]", "<ol>", $text);
    $text = str_replace("[/ol]", "</ol>", $text);
    
    // paragraphs
    $text = str_replace("[p]", "<p>", $text);
    $text = str_replace("[/p]", "</p>", $text);
    
    // links
    $text = str_replace("[a href='", "<a href='", $text);
    $text = str_replace("[/a]", "</a>", $text);
    
    // mathjax:
    // nothing to do: <math> ... </math> set as mathjax inline marker. (?!)
    
    return $text;
  }
}

?>
