// from http://www.siteduzero.com/tutoriel-3-34703-insertion-de-balises-dans-une-zone-de-texte.html
// (in french)
function insertTag(startTag, endTag, textAreaId) {
  
  var textArea = document.getElementById(textAreaId);
  var scroll = textArea.scrollTop;
  var currentSelection;
  
  if (scroll == null) return; // no focus on textArea
  textArea.focus();
  
  if (window.ActiveXObject) { // internet explorer
    var textRange = document.selection.createRange();            
    currentSelection = textRange.text;
    
    textRange.text = startTag + currentSelection + endTag;
    textRange.moveStart("character", -endTag.length - currentSelection.length);
    textRange.moveEnd("character", -endTag.length);
    textRange.select();     
  }
  else { // other browsers
    var startSelection = textArea.value.substring(0, textArea.selectionStart);
    currentSelection = textArea.value.substring(textArea.selectionStart, textArea.selectionEnd);
    var endSelection = textArea.value.substring(textArea.selectionEnd);
    
    textArea.value = startSelection + startTag + currentSelection + endTag + endSelection;
    textArea.focus();
    textArea.setSelectionRange(startSelection.length + startTag.length, startSelection.length + startTag.length + currentSelection.length);
  } 
  
  textArea.scrollTop = scroll; // scroll is redefined
}

function formatText(button, textAreaId, adminMode, param) {
  
  var src, dotIndex, fileType; // for image, audio, video...
  
  switch (button) {
    
    // basic controls
    // ==============
    
    case "bold":
      if (adminMode) insertTag("<b>", "</b>", textAreaId);
      else insertTag("[b]", "[/b]", textAreaId);
      break;
    
    case "italic":
      if (adminMode) insertTag("<i>", "</i>", textAreaId);
      else insertTag("[i]", "[/i]", textAreaId);
      break;

    case "underline":
      if (adminMode) insertTag("<u>", "</u>", textAreaId);
      else insertTag("[u]", "[/u]", textAreaId);
      break;

    case "strike_through":
      if (adminMode) insertTag("<s>", "</s>", textAreaId);
      else insertTag("[s]", "[/s]", textAreaId);
      break;

    // special: font size [only admin mode]
    case "font_size":
      // subcases: tiny, small, large, huge
      switch (param) {
        case "tiny":
          insertTag("<tiny>", "</tiny>", textAreaId);
          break;
        case "small":
          insertTag("<small>", "</small>", textAreaId);
          break;
        case "large":
          insertTag("<large>", "</large>", textAreaId);
          break;
        case "huge":
          insertTag("<huge>", "</huge>", textAreaId);
          break;
        }
        break;

    // special: title [only admin mode]
    case "title":
      // subcases: h1 to h6
      var titleLevel = param[1];
      insertTag("<h" + titleLevel + ">", "</h" + titleLevel + ">", textAreaId);
      break;

    case "line_jump":
      if (adminMode) insertTag("<br>", "", textAreaId);
      else insertTag("[br]", "", textAreaId);
      break;

    case "horizontal_line":
      if (adminMode) insertTag("<hr>", "", textAreaId);
      else insertTag("[hr]", "", textAreaId);
      break;


    // format blocks of text
    // ==============
    
    case "paragraph":
      if (adminMode) insertTag("<p>", "</p>", textAreaId);
      else insertTag("[p]", "[/p]", textAreaId);
      break;
    
    case "indent":
      if (adminMode) insertTag("<indent>", "</indent>", textAreaId);
      else insertTag("[indent]", "[/indent]", textAreaId);
      break;
    
    case "align_left":
      if (adminMode) insertTag("<left>", "</left>", textAreaId);
      else insertTag("[left]", "[/left]", textAreaId);
      break;
    
    case "align_center":
      if (adminMode) insertTag("<center>", "</center>", textAreaId);
      else insertTag("[center]", "[/center]", textAreaId);
      break;
    
    case "align_right":
      if (adminMode) insertTag("<right>", "</right>", textAreaId);
      else insertTag("[right]", "[/right]", textAreaId);
      break;
    
    case "boxed": // only admin mode
      insertTag("<box>", "</box>", textAreaId);
      break;
    
    case "color":
      if (adminMode) insertTag("<'" + param + "'>", "</#>", textAreaId);
      else insertTag("['" + param + "'>", "[/#]", textAreaId);
      break;
    
    
    // lists
    // =====
    
    // TODO: when something is selected, also add <li> elements
    case "list_element":
      if (adminMode) insertTag("<li>", "</li>", textAreaId);
      else insertTag("[li]", "[/li]", textAreaId);
      break;
    
    case "bulleted_list":
      if (adminMode) insertTag("<ul>", "</ul>", textAreaId);
      else insertTag("[ul]", "[/ul]", textAreaId);
      break;
      
    case "numbered_list":
      if (adminMode) insertTag("<ol>", "</ol>", textAreaId);
      else insertTag("[ol]", "[/ol]", textAreaId);
      break;
      
    
    // misc, including multimedia
    // ==========================
    
    case "code":
      // subcases: normal, encadre, numLines, numLines+encadre
      switch (param) {
        case "plain":
          if (adminMode) insertTag("<code>", "</code>", textAreaId);
          else insertTag("[code]", "[/code]", textAreaId);
          break;
        case "boxed":
          if (adminMode) insertTag("<codeb>", "</codeb>", textAreaId);
          else insertTag("[codeb]", "[/codeb]", textAreaId);
          break;
        case "numline":
          if (adminMode) insertTag("<codel>", "</codel>", textAreaId);
          else insertTag("[codel]", "[/codel]", textAreaId);
          break;
        case "boxed_numline":
          if (adminMode) insertTag("<codebl>", "</codebl>", textAreaId);
          else insertTag("[codebl]", "[/codebl]", textAreaId);
          break;
        }
        break;
      
    case "link":
      var link = typeof(param) === 'undefined'
        ? prompt("URL:", "http://")
        : param; // param is given if insert from files list
      if (adminMode) insertTag("<a href='" + link + "'>", "</a>", textAreaId);
      else insertTag("[a href='" + link + "'>", "[/a]", textAreaId);
      break;
      
    // next three : only in admin mode
      
    case "image":
      // TODO: browse images, more friendly way to set width, height, alt...
      src = typeof(param) === 'undefined'
        ? prompt("URL:", "http://")
        : param;
      // auto-closing: no end tag
      insertTag("<img src='" + src + "' width=200 height=200 alt='image description'/>", "", textAreaId);
      break;
      
    case "audio":
      // TODO: browse audio files, find a friendly way to ask for multi-sources (?!)
      src = typeof(param) === 'undefined'
        ? prompt("URL:", "http://")
        : param;
      // parse source to know file type
      dotIndex = src.lastIndexOf('.');
      fileType = src.substring(dotIndex, src.length);
      insertTag("<audio controls='controls'><source src='" + src + "' type=audio/'" + fileType + "'/>", "</audio>", textAreaId);
      break;
    
    // see http://www.alsacreations.com/article/lire/1125-introduction-balise-video-html5-mp4-h264-webm-ogg-theora.html
    // (in french)
    case "video":
      // TODO: browse video files, find a friendly way to ask for multi-sources (?!)
      src = typeof(param) === 'undefined'
        ? prompt("URL:", "http://")
        : param;
      // parse source to know file type
      dotIndex = src.lastIndexOf('.');
      fileType = src.substring(dotIndex, src.length);
      if (fileType == "ogg") fileType = "ogv"; // special case (?!)
      insertTag("<video width=200 height=200 controls='controls'><source src='" + src + "' type=video/'" + fileType + "'/>", "</video>", textAreaId);
      break;
  }
}

// specials: some choices (colors, titles, font sizes)
function toggleEditorChoices(choicesName) {
  
  var divName = "";
  switch (choicesName) {
    case "font_size":
      divName = "fontSizes";
      break;
    case "title":
      divName = "titleLevels";
      break;
    case "color":
      divName = "colorPalette";
      break;
    case "code":
      divName = "codeStyles";
      break;
  }
  
  var choices = document.getElementById(divName);
  //if (choices.style.display != "inline") choices.style.display = "inline";
  if (choices.style.display != "block") choices.style.display = "block";
  else choices.style.display = "none";
}
